/* 
   Unix SMB/Netbios implementation.
   Version 1.9.
   Safe string handling routines.
   Copyright (C) Andrew Tridgell 1994-1998
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _SAFE_STRING_H
#define _SAFE_STRING_H

#undef strcpy
#define strcpy(dest,src) __ERROR__XX__NEVER_USE_STRCPY___;

#undef strcat
#define strcat(dest,src) __ERROR__XX__NEVER_USE_STRCAT___;

#undef sprintf
#define sprintf __ERROR__XX__NEVER_USE_SPRINTF__;

#if defined(__GNUC__) && __GNUC__ >= 2 && defined(__OPTIMIZE__)
extern void __ERROR__pstrcpy_to_fstring(void), __ERROR__pstrcpy_to_odd_size(void);
extern void __ERROR__pstrcat_to_fstring(void), __ERROR__pstrcat_to_odd_size(void);
extern void __ERROR__fstrcpy_to_pstring(void), __ERROR__fstrcpy_to_odd_size(void);
extern void __ERROR__fstrcat_to_pstring(void), __ERROR__fstrcat_to_odd_size(void);
#define pstrcpy(d,s) ({ \
	switch(sizeof(d)) { \
		case sizeof(char *): case sizeof(pstring): break; \
		case sizeof(fstring): __ERROR__pstrcpy_to_fstring(); break; \
		default: __ERROR__pstrcpy_to_odd_size(); break; \
	} \
	safe_strcpy((d),(s),sizeof(pstring)); })
#define pstrcat(d,s) ({ \
	switch(sizeof(d)) { \
		case sizeof(char *): case sizeof(pstring): break; \
		case sizeof(fstring): __ERROR__pstrcat_to_fstring(); break; \
		default: __ERROR__pstrcat_to_odd_size(); break; \
	} \
	safe_strcat((d),(s),sizeof(pstring)); })
#define fstrcpy(d,s) ({ \
	switch(sizeof(d)) { \
		case sizeof(char *): case sizeof(fstring): break; \
		case sizeof(pstring): __ERROR__fstrcpy_to_pstring(); break; \
		default: __ERROR__fstrcpy_to_odd_size(); break; \
	} \
	safe_strcpy((d),(s),sizeof(fstring)); })
#define fstrcat(d,s) ({ \
	switch(sizeof(d)) { \
		case sizeof(char *): case sizeof(fstring): break; \
		case sizeof(pstring): __ERROR__fstrcat_to_pstring(); break; \
		default: __ERROR__fstrcat_to_odd_size(); break; \
	} \
	safe_strcat((d),(s),sizeof(fstring)); })
#else
#define pstrcpy(d,s) safe_strcpy((d),(s),sizeof(pstring))
#define pstrcat(d,s) safe_strcat((d),(s),sizeof(pstring))
#define fstrcpy(d,s) safe_strcpy((d),(s),sizeof(fstring))
#define fstrcat(d,s) safe_strcat((d),(s),sizeof(fstring))
#endif

#define wpstrcpy(d,s) safe_strcpy_w((d),(s),sizeof(wpstring))
#define wpstrcat(d,s) safe_strcat_w((d),(s),sizeof(wpstring))
#define wfstrcpy(d,s) safe_strcpy_w((d),(s),sizeof(wfstring))
#define wfstrcat(d,s) safe_strcat_w((d),(s),sizeof(wfstring))

#endif
