/* 
 *  Unix SMB/Netbios implementation.
 *  Version 1.9.
 *  RPC Pipe client / server routines
 *  Copyright (C) Luke Kenneth Casson Leighton 1996-2001
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* MKPROTO: rpc_parse_proto.h		(do not remove this line) */


#include "includes.h"
#include "tng_misc.h"
#include "libsamba.h"
#include "libsmb.h"
#include "rpc_parse.h"


void copy_nt_creds(struct ntuser_creds *to,
				const struct ntuser_creds *from)
{
	if (from == NULL)
	{
		DEBUG(10,("copy_nt_creds: null creds\n"));
		to->domain[0] = 0;
		to->user_name[0] = 0;
		pwd_set_nullpwd(&to->pwd);
		to->ntlmssp_flags = 0;

		return;
	}
	fstrcpy(to->domain, from->domain);
	fstrcpy(to->user_name, from->user_name);
	to->pwd = from->pwd;
	to->ntlmssp_flags = from->ntlmssp_flags;
	DEBUG(10,("copy_nt_creds: user %s domain %s nopw %s flgs: %x\n",
	       to->user_name, to->domain, BOOLSTR(pwd_is_nullpwd(&to->pwd)),
	       to->ntlmssp_flags));
}

/*******************************************************************
reads or writes a structure.
********************************************************************/
BOOL creds_io_cmd(char *desc, CREDS_CMD *r_u, prs_struct *ps, int depth)
{
	if (r_u == NULL) return False;

	prs_debug(ps, depth, desc, "creds_io_cmd");
	depth++;

	prs_align(ps);

	prs_uint16("version ", ps, depth, &(r_u->version));
	prs_uint16("command ", ps, depth, &(r_u->command));

	prs_string("pipename", ps, depth,   r_u->pipename, strlen(r_u->pipename), sizeof(r_u->pipename));
	prs_string("hostname", ps, depth,   r_u->hostname, strlen(r_u->hostname), sizeof(r_u->hostname));
	prs_align(ps);
	
	prs_uint32("ptr_vuser", ps, depth, &(r_u->ptr_vuser));
	if (r_u->ptr_vuser != 0)
	{
		if (UNMARSHALLING(ps))
		{
			r_u->vuser = g_new0(user_struct, 1);
		}
		if (!vuid_io_user_struct("vuser", r_u->vuser, ps, depth))
		{
			return False;
		}
	}

	return True;
}


BOOL create_user_creds( prs_struct *ps,
				const char* pipe_name, 
				const char* client_host_name, 
				uint16 version, uint16 command,
				const user_struct *usr)
{
	CREDS_CMD cmd;

	ZERO_STRUCT(cmd);

	DEBUG(10,("create_user_creds: ncacn_np:%s[%s] %d %d\n",
		client_host_name, pipe_name, version, command));

	fstrcpy(cmd.pipename, pipe_name);
	fstrcpy(cmd.hostname, client_host_name);
	cmd.version = version;
	cmd.command = command;
	cmd.ptr_vuser = usr != NULL ? 1 : 0;
	/* safe, because we are marshalling, so cmd should not be modified */
	cmd.vuser = STRIP_CONST(usr);

	prs_init(ps, MARSHALL);

	ps->offset = 4;
	return creds_io_cmd("creds", &cmd, ps, 0);
}

