
/* 
 *  Unix SMB/Netbios implementation.
 *  Version 1.9.
 *  RPC Pipe client / server routines
 *  Copyright (C) Luke Kenneth Casson Leighton 1996-2001
 *  Copyright (C) Paul Ashton                  1997.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "includes.h"
#include "debug.h"
#include "rpc_parse.h"
#include "others.h"


/*******************************************************************
turns a DCE/RPC request into a DCE/RPC reply
********************************************************************/
static BOOL api_noauth_create_pdu(rpcsrv_struct * l, uint32 data_start,
				  prs_struct *resp)
{
	BOOL ret;
	uint32 data_len;
	uint32 frag_len;
	uint32 data_end = l->rdata.offset;
	RPC_HDR_RESP hdr_resp;
	uint8 flags = 0x0;
	uint32 call_id = l->hdr.call_id;

	DEBUG(5,
	      ("create_noauth_reply: data_start: %d data_end: %d max_tsize: %d\n",
	       data_start, data_end, l->hdr_ba.bba.max_tsize));

	if (l->hdr.auth_len != 0)
	{
		return False;
	}

	prs_init(resp, MARSHALL);

	/* set up rpc header (fragmentation issues) */
	if (data_start == 0)
	{
		flags = RPC_FLG_FIRST;
	}
	else
	{
		flags = 0;
	}

	hdr_resp.alloc_hint = data_end - data_start;	/* calculate remaining data to be sent */
	hdr_resp.cancel_count = 0x0;
	hdr_resp.context_id = l->hdr_req.context_id;
	hdr_resp.reserved = 0x0;

	DEBUG(10, ("alloc_hint: %d\n", hdr_resp.alloc_hint));

	if (hdr_resp.alloc_hint + 0x18 <= l->hdr_ba.bba.max_tsize)
	{
		flags |= RPC_FLG_LAST;
		frag_len = hdr_resp.alloc_hint + 0x18;
	}
	else
	{
		frag_len = l->hdr_ba.bba.max_tsize;
	}

	data_len = frag_len - 0x18;

	prs_set_start(resp, 0);
	resp->end = 0x18;

	if (!make_rpc_hdr(&l->hdr, RPC_RESPONSE, flags, call_id, frag_len, 0))
	{
		return False;
	}

	DEBUG(10, ("hdr flags: %x\n", flags));

	/* store the header in the data stream */
	smb_io_rpc_hdr("rhdr", &(l->hdr), resp, 0);
	smb_io_rpc_hdr_resp("resp", &(hdr_resp), resp, 0);

	/* don't use rdata: use rdata_i instead, which moves... */
	/* make a pointer to the rdata data, NOT A COPY */

	ret = prs_append_data(resp, prs_data(&l->rdata, data_start), data_len);
	l->rdata_offset += data_len;

	prs_debug_out(resp, "resp", 200);
	prs_debug_out(&l->rdata, "rdata", 200);

	return ret;
}

static BOOL api_noauth_auth_gen(rpcsrv_struct * l, prs_struct *resp,
				enum RPC_PKT_TYPE pkt_type)
{
	BOOL ret;

	DEBUG(10, ("api_noauth_auth_gen\n"));

	prs_init(resp, MARSHALL);

	make_rpc_hdr(&l->hdr, pkt_type, RPC_FLG_FIRST | RPC_FLG_LAST,
		     l->hdr.call_id, l->rdata.offset + 0x10, 0);

	smb_io_rpc_hdr("", &l->hdr, resp, 0);

	ret = prs_append_prs(resp, &l->rdata);
	prs_free_data(&l->rdata);

	return ret;
}

static BOOL api_noauth_auth_chk(rpcsrv_struct * l, enum RPC_PKT_TYPE pkt_type)
{
	uint16 vuid = get_sec_ctx();
	l->auth_validated = True;

	if (vuid == UID_FIELD_INVALID)
	{
		DEBUG(5,("api_noauth_auth_chk: no vuid\n"));
		return False;
	}

	return True;
}

srv_auth_fns noauth_fns = {
	NULL,
	api_noauth_auth_chk,
	api_noauth_auth_gen,
	NULL,
	api_noauth_create_pdu,
};
