/* 
   Unix SMB/Netbios implementation.
   Version 1.9.
   NT Domain Authentication SMB / MSRPC client
   Copyright (C) Luke Kenneth Casson Leighton 1996-2001
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "includes.h"
#include "libsamba.h"
#include "libsmb.h"
#include "rpc_parse.h"
#include "rpc_client.h"
#include "rpcclient.h"
#include "nterr.h"

extern struct ntuser_creds *usr_creds;

extern FILE *out_hnd;

#if 0
/****************************************************************************
experimental nt login trust account change.
****************************************************************************/
NTSTATUS cmd_netlogon_pwset(struct client_info *info, int argc, char *argv[])
{
	NTSTATUS ret;
	fstring domain;
	fstring acct_name;
	fstring name;
	fstring sid;
	DOM_SID sid1;
	uint32 user_rid;
	int opt;
	char *password = NULL;
	pstring upwb;
	int plen = 0;
	int len = 0;
	UNISTR2 upw;
	uchar ntpw[16];
	STRING2 secret;

	POLICY_HND lsa_pol;
	POLICY_HND pol_sec;

	uchar old_trust_passwd[16];
	char *p;
	uint16 validation_level;

	fstring wks_name;
	fstring srv_name;

	fstrcpy(srv_name, "\\\\");
	fstrcat(srv_name, info->dest_host);
	strupper(srv_name);

	fstrcpy(wks_name, "\\\\");
	if (strequal(srv_name, "\\\\.") &&
	    strequal(info->dest_host, info->myhostname))
	{
		fstrcat(wks_name, ".");
	}
	else
	{
		fstrcat(wks_name, info->dest_host);
	}
	strupper(wks_name);

	domain[0] = 0;
	if (usr_creds != NULL)
	{
		fstrcpy(domain, usr_creds->domain);
	}

	if (domain[0] == 0)
	{
		fstrcpy(domain, info->dom.level3_dom);
	}

	argc--;
	argv++;

	if (domain[0] == 0)
	{
		report(out_hnd, "no domain specified.\n");
	}

	DEBUG(5, ("do_nt_login_test: username %s from: %s\n",
		  nt_user_name, info->myhostname));

	fstrcpy(acct_name, argv[0]);
	len = strlen(acct_name) - 1;
	if (acct_name[len] == '$')
	{
		memcpy(name, argv[0], len);
		name[len] = 0;
	}

	ret = cli_nt_setup_creds(srv_name, domain, info->myhostname,
				 trust_acct, old_trust_passwd,
				 SEC_CHAN_WKSTA, &validation_level);
	if (!NT_STATUS_IS_OK(ret))
		return ret;

	/*
	 * ok.  this looks really weird, but if you don't open
	 * the connection to the workstation first, then the
	 * set trust account on the SAM database may get the
	 * local copy-of trust account out-of-sync with the
	 * remote one, and you're stuffed!
	 */
	if (!lsa_open_policy(wks_name, &lsa_pol, True, 0x02000000))
	{
		report(out_hnd, "Connection to %s FAILED\n", wks_name);
		report(out_hnd, "(Do a \"use \\\\%s -U localadmin\")\n",
		       wks_name);
		return NT_STATUS_UNSUCCESSFUL;
	}
	if (!lsa_open_secret(&lsa_pol, "$MACHINE.ACC", 0x020003, &pol_sec))
	{
		lsa_close(&lsa_pol);
		report(out_hnd, "Open Secret failed\n");
		return NT_STATUS_UNSUCCESSFUL;
	}

	if (!lsa_query_secret(&pol_sec, &secret, NULL) ||
	    !secret_to_nt_owf(&old_trust_passwd, &secret))
	{
		ret = NT_STATUS_UNSUCCESSFUL;
		report(out_hnd, "Query local Trust Account password: Failed\n");
		goto out;
	}

	/*
	 * generate new random password.  unicode string is stored
	 * in secret $MACHINE.ACC; nt owf is sent in net_srv_pwset.
	 */

	upw.uni_str_len = 0xc;
	upw.uni_max_len = 0xc;
	password = (char *)upw.buffer;
	plen = upw.uni_str_len * 2;
	generate_random_buffer(password, plen, True);

	nt_owf_genW(&upw, ntpw);

	ret = net_srv_pwset(srv_name, &sid1,
			    acct_name, password, plen, &user_rid);
	if (!NT_STATUS_IS_OK(ret))
	{
		report(out_hnd, "Set remote Trust Account password: Failed\n");
		goto out;
	}

	report(out_hnd, "Set remote Trust Account password: OK\n");

	strupper(domain);
	strupper(name);

	/* valid pol_sec on $MACHINE.ACC, set trust passwd */
	secret_store_data(&secret, password, plen);

	ret = lsa_set_secret(&pol_sec, &secret);
	if (NT_STATUS_IS_OK(ret))
	{
		report(out_hnd, "Set $MACHINE.ACC: OK\n");
		report(out_hnd, "cmd_nt_login: login (%s) test succeeded\n",
		       nt_user_name);
	}
	else
	{
		report(out_hnd, "Set $MACHINE.ACC: FAILED\n");
	}

out:
	ZERO_STRUCTP(upw);
	ZERO_ARRAY(ntpw);
	lsa_close(&pol_sec);
	lsa_close(&lsa_pol);

	return ret;
}

#endif


/****************************************************************************
experimental nt trusted domain list.
****************************************************************************/
NTSTATUS cmd_netlogon_dom_list(struct client_info *info, int argc, char *argv[])
{
	NTSTATUS status;
	BUFFER2 buf;

	fstring srv_name;

	fstrcpy(srv_name, "\\\\");
	fstrcat(srv_name, info->dest_host);
	strupper(srv_name);

	ZERO_STRUCT(buf);

	status = cli_net_trust_dom_list(srv_name, &buf);
	if (NT_STATUS_IS_OK(status))
	{
		fstring domains;
		buffer2_to_multistr(domains, &buf, sizeof(domains));
		report(out_hnd, "trusted_dom_list: (%s)\n", domains);
	}

	misc_buffer2_free(&buf);

	return status;
}


/*
 * "ntlogin":
 */
static void display_net_user_info_3(const NET_USER_INFO_3* usr)
{
	uint32 i;

	if (! usr)
		return;
	show_unistr2("\tLogon Domain:      %s\n", &usr->uni_logon_dom);
	show_unistr2("\tLogon Server:      %s\n", &usr->uni_logon_srv);
	show_unistr2("\tUsername:          %s\n", &usr->uni_user_name);
	show_unistr2("\tFull name:         %s\n", &usr->uni_full_name);
	show_unistr2("\tLogonscript:       %s\n", &usr->uni_logon_script);
	show_unistr2("\tProfile:           %s\n", &usr->uni_profile_path);
	show_unistr2("\tHome Drive:        %s\n", &usr->uni_dir_drive);
	show_unistr2("\tHomedir:           %s\n", &usr->uni_home_dir);

	fprintf(out_hnd, "\tUser RID:          %d (0x%x)\n", usr->user_id, usr->user_id);
	fprintf(out_hnd, "\tPrimary Group RID: %d (0x%x)\n", usr->group_id, usr->group_id);
	fprintf(out_hnd, "\tUser Flags:        0x%x\n", usr->user_flgs);

	fprintf(out_hnd, "\tGroups:\n");
	for (i = 0; i < usr->num_groups; i++)
	{
		fprintf(out_hnd, "\t\t%d (0x%x), 0x%x\n", usr->gids[i].g_rid, usr->gids[i].g_rid, usr->gids[i].attr);
	}
	fprintf(out_hnd, "\tNumber of other SIDs: %d\n", usr->num_other_sids);
}

NTSTATUS cmd_netlogon_login_test(struct client_info *info, int argc, char *argv[])
{
	NTSTATUS status = NT_STATUS_UNSUCCESSFUL;
	struct cli_connection *con = NULL;
	int opt;
	fstring nt_user_name;
	char *nt_password;
	uchar trust_passwd[16];
	uchar nt_pw[16];
	uchar lm_pw[16];
	fstring trust_acct;
	uint16 sec_chan = SEC_CHAN_WKSTA;
	fstring domain;
	char *p;
	uint16 validation_level;

	fstring wks_name;
	fstring srv_name;
	const char *member_domain = domain;
	const char *asc_trust_pw = NULL;

	fstrcpy(srv_name, "\\\\");
	fstrcat(srv_name, info->dest_host);
	strupper(srv_name);

	fstrcpy(wks_name, "\\\\");
	if (strequal(srv_name, "\\\\.") &&
	    strequal(info->dest_host, info->myhostname))
	{
		fstrcat(wks_name, ".");
	}
	else
	{
		fstrcat(wks_name, info->dest_host);
	}
	strupper(wks_name);

	domain[0] = 0;
	if (usr_creds != NULL)
	{
		fstrcpy(domain, usr_creds->domain);
	}

	if (domain[0] == 0)
	{
		fstrcpy(domain, info->dom.level3_dom);
	}
#if 0
	/* trust account passwords */
	pstring new_mach_pwd;

	/* initialisation */
	new_mach_pwd[0] = 0;
#endif

	argc--;
	argv++;

	if (argc < 1)
	{
		cmd_help_current(info);
		return NT_STATUS_INVALID_PARAMETER;
	}

	fstrcpy(nt_user_name, argv[0]);

	p = strchr(nt_user_name, '\\');
	if (p != NULL)
	{
		fstrcpy(domain, nt_user_name);
		p = strchr(domain, '\\');
		if (p != NULL)
		{
			*p = 0;
			fstrcpy(nt_user_name, p + 1);
		}

	}

	if (domain[0] == 0)
	{
		report(out_hnd, "no domain specified.\n");
	}

	argc--;
	argv++;

	if (argc > 0)
	{
		nt_password = argv[0];
		argc--;
		argv++;
	}
	else
	{
		nt_password = getpass("Enter NT Login password:");
	}

	nt_lm_owf_gen(nt_password, nt_pw, lm_pw);


	argc++;
	argv--;
	while ((opt = getopt(argc, argv, "d:p:i")) != EOF)
	{
		switch (opt)
		{
			case 'd':
				member_domain = optarg;
				break;
			case 'p':
				asc_trust_pw = optarg;
				break;
			case 'i':
				sec_chan = SEC_CHAN_DOMAIN;
				break;
		}
	}

	DEBUG(5, ("do_nt_login_test: username %s from: %s\n",
		  nt_user_name, info->myhostname));

	fstrcpy(trust_acct, info->myhostname);
	fstrcat(trust_acct, "$");

	if (asc_trust_pw)
	{
		/* trust pw specified on cmd line */
		uchar tmp_lm_pw[16];
		nt_lm_owf_gen(asc_trust_pw, trust_passwd, tmp_lm_pw);
	}
	else
	{
		if (!msrpc_lsa_query_trust_passwd(wks_name, "$MACHINE.ACC",
						  trust_passwd, NULL))
			goto out;
	}

	status = cli_nt_setup_creds(srv_name, member_domain, info->myhostname,
				    trust_acct, trust_passwd,
				    sec_chan, &validation_level, &con);
	if (!NT_STATUS_IS_OK(status))
		goto out;

	/* do an NT login */
	status = cli_nt_login_interactive(srv_name, info->myhostname,
					  domain, nt_user_name,
					  getuid(), lm_pw, nt_pw,
					  &info->dom.ctr,
					  validation_level,
					  &info->dom.user_info3);
	if (!NT_STATUS_IS_OK(status))
		goto out;

	/* ok!  you're logged in!  do anything you like, then... */
	display_net_user_info_3(&info->dom.user_info3);

	/* do an NT logout */
	if (! cli_nt_logoff(srv_name, info->myhostname, &info->dom.ctr))
		report(out_hnd, "INFO: logoff failed\n");

	report(out_hnd, "cmd_nt_login: login (%s): %s\n",
	       nt_user_name, get_nt_error_msg(status));

out:
	ZERO_ARRAY(trust_passwd);
	ZERO_ARRAY(lm_pw);
	ZERO_ARRAY(nt_pw);
	cli_connection_unlink(con);

	return status;
}


NTSTATUS cmd_netlogon_domain_test(struct client_info *info, int argc, char *argv[])
{
	NTSTATUS ret = NT_STATUS_UNSUCCESSFUL;
	char *nt_trust_dom;
	uchar trust_passwd[16];
	fstring inter_dom_acct;
	fstring trust_sec_name;
	fstring domain;
	uint16 validation_level;

	fstring wks_name;
	fstring srv_name;

	fstrcpy(srv_name, "\\\\");
	fstrcat(srv_name, info->dest_host);
	strupper(srv_name);

	fstrcpy(wks_name, "\\\\");
	if (strequal(srv_name, "\\\\.") &&
	    strequal(info->dest_host, info->myhostname))
	{
		fstrcat(wks_name, ".");
	}
	else
	{
		fstrcat(wks_name, info->dest_host);
	}
	strupper(wks_name);

	domain[0] = 0;
	if (usr_creds != NULL)
	{
		fstrcpy(domain, usr_creds->domain);
	}

	if (domain[0] == 0)
	{
		fstrcpy(domain, info->dom.level3_dom);
	}

	if (argc < 2)
	{
		report(out_hnd, "domtest: must specify domain name\n");
		return NT_STATUS_INVALID_PARAMETER;
	}

	nt_trust_dom = argv[1];

	DEBUG(5, ("do_nt_login_test: domain %s\n", nt_trust_dom));

	fstrcpy(trust_sec_name, "G$$");
	fstrcat(trust_sec_name, nt_trust_dom);
	strupper(inter_dom_acct);

	fstrcpy(inter_dom_acct, nt_trust_dom);
	fstrcat(inter_dom_acct, "$");

	if (msrpc_lsa_query_trust_passwd(wks_name, trust_sec_name,
					 trust_passwd, NULL) &&
	    cli_nt_setup_creds(srv_name, domain,
			       info->myhostname, inter_dom_acct,
			       trust_passwd, SEC_CHAN_DOMAIN,
			       &validation_level, NULL))
	{
		ret = NT_STATUS_OK;
		report(out_hnd,
		       "cmd_nt_login: credentials (%s) test succeeded\n",
		       nt_trust_dom);
	}

	ZERO_ARRAY(trust_passwd);
	return ret;
}


/*
 * "samsync":
 */
NTSTATUS cmd_sam_sync(struct client_info *info, int argc, char *argv[])
{
	uchar trust_passwd[16];
	fstring trust_acct;
	fstring domain;

	fstring wks_name;
	fstring srv_name;
	uint32 database_id = 0;

	fstrcpy(srv_name, "\\\\");
	fstrcat(srv_name, info->dest_host);
	strupper(srv_name);

	fstrcpy(domain, info->dom.level5_dom);

	fstrcpy(wks_name, "\\\\");
#if 0
	if (strequal(srv_name, "\\\\."))
#endif
	{
		fstrcat(wks_name, ".");
	}
#if 0
	else
	{
		fstrcat(wks_name, info->myhostname);
	}
#endif
	strupper(wks_name);

	if (!get_dc_name(domain, srv_name, 0x1b))
	{
		report(out_hnd, "could not locate server for domain %s\n",
		       domain);
		return NT_STATUS_DOMAIN_CONTROLLER_NOT_FOUND;
	}

	fstrcpy(trust_acct, info->myhostname);
	fstrcat(trust_acct, "$");

	if (!msrpc_lsa_query_trust_passwd(wks_name, "$MACHINE.ACC",
					  trust_passwd, NULL))
	{
		report(out_hnd, "cmd_sam_sync: no trust account password\n");
		return NT_STATUS_ACCESS_DENIED;
	}

	if (argc >= 2)
		database_id = atoi(argv[1]);

	return net_sam_sync(srv_name, domain, info->myhostname,
			    database_id,
			    trust_acct, trust_passwd,
			    display_sam_sync, out_hnd);
}
