/* 
   Unix SMB/Netbios implementation.
   Version 1.9.
   Samba utility functions
   Copyright (C) Andrew Tridgell 1992-1999
   Copyright (C) Luke Kenneth Casson Leighton 1996 - 1999
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "includes.h"
#include "libsamba.h"
#include "libsmb.h"
#include "nterr.h"
#include "rpcclient.h"


/****************************************************************************
 display sam sync structure
 ****************************************************************************/
static void display_sam_sync_ctr(FILE *out_hnd,
				const SAM_DELTA_HDR *delta,
				const SAM_DELTA_CTR *ctr)
{
	fstring name;
	uint32 i;

	report(out_hnd, "\ntarget_rid: %08x\n", delta->target_rid);

	switch (delta->type)
	{
		case 1:
		{
			unistr2_to_ascii(name, &(ctr->domain_info.uni_dom_name), sizeof(name)-1); 
			report(out_hnd, "Domain: %s\n", name);
			break;
		}
		case 2:
		{
			show_unistr2("cn: %s\n", &ctr->group_info.uni_grp_name);
			show_unistr2("description: %s\n", &ctr->group_info.uni_grp_desc);
			break;
		}
		case 5:
		{
			unsigned char lm_pwd[16];
			unsigned char nt_pwd[16];
			fstring tmp;

			show_unistr2("ntuid: %s\n", &ctr->account_info.uni_acct_name);
			report(out_hnd, "rid: %08x\n", ctr->account_info.user_rid);
			show_unistr2("cn: %s\n", &ctr->account_info.uni_full_name);

			sam_pwd_hash(ctr->account_info.user_rid, ctr->account_info.pass.buf_lm_pwd, lm_pwd, 0);
			pwdb_sethexpwd(tmp, lm_pwd, 0);
			report(out_hnd, "lmPassword: %s\n", tmp);

			sam_pwd_hash(ctr->account_info.user_rid, ctr->account_info.pass.buf_nt_pwd, nt_pwd, 0);
			pwdb_sethexpwd(tmp, nt_pwd, 0);
			report(out_hnd, "ntPassword: %s\n", tmp);
			break;
		}
		case 8:
		{
			const SAM_GROUP_MEM_INFO *grp = &ctr->grp_mem_info;

			for (i = 0; i < grp->num_members; i++)
			{
				report(out_hnd, "member_rid: %08x\n",
				       grp->rids[i]);
			}
			break;
		}
		case 9:
		{
			show_unistr2("cn: %s\n", &ctr->alias_info.uni_als_name);
			show_unistr2("description: %s\n", &ctr->alias_info.uni_als_desc);
			break;
		}
		case 0xc:
		{
			const SAM_ALIAS_MEM_INFO *als = &ctr->als_mem_info;

			for (i = 0; i < als->num_members; i++)
			{
				fstring sidstr;

				sid_to_string(sidstr, &als->sids[i].sid);
				report(out_hnd, "sambaMember: %s\n", sidstr);
			}
			break;
		}
	}
}

NTSTATUS display_sam_sync(uint32 database_id,
			  const SAM_DELTA_HDR *delta,
			  const SAM_DELTA_CTR *ctr,
			  void *file)
{
	display_sam_sync_ctr(file, delta, ctr);
	return NT_STATUS_OK;
}
